local ShowMyDPS = LibStub("AceAddon-3.0"):NewAddon("ShowMyDPS", "AceConsole-3.0", "AceEvent-3.0")

DBDefault = {
	char = {
		handleDPS = true,
		isLock = false,
		isShown = true,
		oocHide = false,
	}
}

function ShowMyDPS:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SMDdb", DBDefault)
	self:RegisterChatCommand("smd","SlashHandler")
end

function ShowMyDPS:OnEnable()
	self:CreateFrame()
	self:RegisterEvent("PLAYER_REGEN_ENABLED","LeaveCombat")
	self:RegisterEvent("PLAYER_REGEN_DISABLED","EnterCombat")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED","AddAmount")
end

function ShowMyDPS:CreateFrame()

	smdFrame = CreateFrame("Frame", "ShowMyDPSFrame", UIParent, "GameTooltipTemplate")
	smdFrame:SetPoint("CENTER", UIParent, self.db.char.x and "BOTTOMLEFT" or "CENTER", self.db.char.x or 0, self.db.char.y or 0)
	smdFrame:EnableMouse(true)
	smdFrame:SetToplevel(true)
	smdFrame:SetMovable(true)
	smdFrame:SetFrameStrata("LOW")
	smdFrame:SetWidth(220)
	smdFrame:SetHeight(25)
	if self.db.char.isShown and not self.db.char.oocHide then
		smdFrame:Show()
	end

	local backdrop_header = {bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile=1, tileSize=16, edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5}}

	smdFrame:SetBackdrop(backdrop_header);
	smdFrame:SetBackdropBorderColor(TOOLTIP_DEFAULT_COLOR.r, TOOLTIP_DEFAULT_COLOR.g, TOOLTIP_DEFAULT_COLOR.b)
	smdFrame:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)

	smdFrame.text = smdFrame:CreateFontString("$parentText", "ARTWORK", "GameFontNormalSmall")
	smdFrame.text:SetPoint("CENTER", smdFrame, "CENTER", 0, 0)
	smdFrame.text:Show()

	smdFrame:SetScript("OnMouseDown", function(frame, button) 
		if not self.db.char.isLock and button ~= "RightButton" then
			frame.isMoving = true
			frame:StartMoving()	
		end
	end)

	smdFrame:SetScript("OnMouseUp", function(frame, button) 
		if not self.db.char.isLock and button ~= "RightButton" then
			if( frame.isMoving ) then
				frame.isMoving = nil
				frame:StopMovingOrSizing()
				self.db.char.x, self.db.char.y = frame:GetCenter()
			end
		else
			self.db.char.handleDPS = not self.db.char.handleDPS
			self:RefreshFrame()
		end
	end)
	self:RefreshFrame()
	ShowMyDPS.Frame = smdFrame
end

function ShowMyDPS:SlashHandler(msg)
	if msg == "lock" then
		self.db.char.isLock = not self.db.char.isLock
	elseif msg == "show" then
		if ShowMyDPS.Frame:IsShown() then ShowMyDPS.Frame:Hide() self.db.char.isShown = false else ShowMyDPS.Frame:Show() self.db.char.isShown = true end
	elseif msg == "oochide" then
		self.db.char.oocHide = not self.db.char.oocHide
		if not InCombatLockdown() then 
			if self.db.char.oocHide then smdFrame:Hide() elseif self.db.char.isShown then smdFrame:Show() end
		end
	elseif msg == "resetpos" then
		self.db.char.x = nil
		self.db.char.y = nil
		self:Printf("You may now reload your ui")
	else
		self:Printf("/smd lock : toggle lock/unlock")
		self:Printf("/smd show : toggle show/hide")
		self:Printf("/smd oochide : toggle hide when out of combat")
		self:Printf("/smd resetpos : resets frame position")
	end
end

local cbtDuration, cbtStart = nil, nil
local cbtDPS, cbtDmgAmount, cbtDmgShown = nil, nil, nil
local cbtHPS, cbtHealAmount, cbtHealShown = nil, nil, nil

function ShowMyDPS:EnterCombat()
	if self.db.char.isShown and self.db.char.oocHide then smdFrame:Show() end
	if cbtStart and cbtStart > 3.6 then
		cbtHealAmount = 0
		cbtDmgAmount = 0
		cbtStart = GetTime() - 3.5
	else
		self:RefreshFrame()
	end
end

function ShowMyDPS:AddAmount(event, ...)
	local arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9, arg10, arg11, arg12, arg13 = ...
	
	if bit.band(arg5, 0x1111) == 0x1111 or bit.band(arg5, 0x511) == 0x511 then -- Checking source
		if arg2 and string.find(arg2, "_DAMAGE") then
			-- Get recount-like feature
			if InCombatLockdown() then
				cbtDmgAmount = cbtDmgAmount or 0
				cbtStart = cbtStart or GetTime() - 3.5
			else
				cbtDmgAmount = 0
				cbtStart = GetTime() - 3.5
			end
			
			-- DPS crap
			if not arg12 then -- Checking dps type
				cbtDmgAmount = cbtDmgAmount + arg9 -- Melee swing
			else
				cbtDmgAmount = cbtDmgAmount + arg12 -- Anything else
			end

			-- Calc
			self:CalculateDmg()
			
			-- Show
			if InCombatLockdown() then
				self:RefreshFrame()
			end
		end
		
		-- Heal crap
		if arg2 and string.find(arg2, "_HEAL") then
			if InCombatLockdown() then
				cbtHealAmount = cbtHealAmount or 0
				cbtStart = cbtStart or GetTime() - 3.5
			else
				cbtHealAmount = 0
				cbtStart = GetTime() - 3.5
			end
			
			cbtHealAmount = cbtHealAmount + ( arg12 - arg13 ) -- Only effective healing
			
			-- Calc
			self:CalculateHeal()
			
			-- Show
			if InCombatLockdown() then
				self:RefreshFrame()
			end
		end
	end
end

function ShowMyDPS:CalculateDmg()
	-- Calculation
	cbtDuration = GetTime() - cbtStart
	if cbtDmgAmount then cbtDPS = (floor((cbtDmgAmount/cbtDuration * 100) + 0.5))/100 end
	if cbtDmgAmount > 1000000 then cbtDmgShown = floor(cbtDmgAmount/1000).."k" else cbtDmgShown = cbtDmgAmount end
end

function ShowMyDPS:CalculateHeal()
	-- Calculation
	cbtDuration = GetTime() - cbtStart
	if cbtHealAmount then cbtHPS = (floor((cbtHealAmount/cbtDuration * 100) + 0.5))/100 end
	if cbtHealAmount > 1000000 then cbtHealShown = floor(cbtHealAmount/1000).."k" else cbtHealShown = cbtHealAmount end
end

function ShowMyDPS:RefreshFrame()
	if self.db.char.handleDPS then
		if cbtDmgAmount and cbtDmgAmount > 0 then
			ShowMyDPSFrameText:SetText(format("%s | %s | %s dps",cbtDmgShown,SecondsToTime(cbtDuration),cbtDPS))
		else
			ShowMyDPSFrameText:SetText("Damage")
		end
	else
		if cbtHealAmount and cbtHealAmount > 0 then
			ShowMyDPSFrameText:SetText(format("%s | %s | %s hps",cbtHealShown,SecondsToTime(cbtDuration),cbtHPS))
		else
			ShowMyDPSFrameText:SetText("Heal")
		end
	end
end

function ShowMyDPS:LeaveCombat()
	self:RefreshFrame()
	if self.db.char.oocHide then smdFrame:Hide() end
	cbtDmgAmount = nil
	cbtHealAmount = nil
	cbtStart = nil
end